////////////////////////////////////////////////////////////////////////////
// File Name:   audio_engine.cpp
// Description: Audio engine
// Author:      Roy Fejgin
// Date:        Autumn 2009
////////////////////////////////////////////////////////////////////////////
#include <list>
#include <assert.h>
#include <math.h>

// STK
#include "Stk.h"
#include "FileWvIn.h"

#include "audio_engine.h"


using namespace std;
using namespace stk;

// Leaky integration factor
#define LEAK            (0.995)

// Invalid envelope value
#define INVALID_ENV     (-1.0)

AudioEngine::AudioEngine(int fftSize, OsMutex* pMutex, char* fileName) :
  m_sampleBuffer(NULL),
  m_timeEnvelope(NULL),
  m_freqEnvelope(NULL),
  m_phaseEnvelope(NULL),
  m_HanningBuffer(NULL),
  m_currSample(0),
  m_numSamples(0),
  m_numWindows(0),
  m_winBufIndex(0),
  m_pMutex(pMutex),
  m_bufferSize(fftSize),
  m_lastRangeStart(0),
  m_lastRangeEnd(1),
  m_lastRangeType(TYPE_TIME)
{

  loadWavFile(fileName);

  // allocate buffers and zero initialize them
  m_WindowedBuffers[0] = new Sample[m_bufferSize];
  memset(m_WindowedBuffers[0], 0, m_bufferSize*sizeof(Sample));
  m_WindowedBuffers[1] = new Sample[m_bufferSize];
  memset(m_WindowedBuffers[1], 0, m_bufferSize*sizeof(Sample));
  m_HanningBuffer = new Sample[m_bufferSize];
  hanning(m_HanningBuffer, m_bufferSize);

  // Create frquency, and phase envelopes
  m_freqEnvelope = new Sample[m_bufferSize/2];
  initEnvelope(m_freqEnvelope, 1.0, m_bufferSize/2);
  m_phaseEnvelope = new Sample[m_bufferSize/2];
  initEnvelope(m_phaseEnvelope, 0, m_bufferSize/2); 
}
void AudioEngine::renderAudio(Sample* audioOutBuf, Sample* audioInBuf, Sample* fftOutBuf, int nFrames)
{
  Sample * pOutputSample = (Sample *) audioOutBuf;
  Sample * pSample = (Sample *) fftOutBuf;
  static float lastTimeEnvelopeVal = 1.0;
  static float lastFreqEnvelopeVal = 1.0;

  memset(audioOutBuf, 0, sizeof(Sample)*nFrames);
  assert(nFrames == m_bufferSize/2);

  int readCount = m_bufferSize;

  // Lock, since the sample buffer may be changed when the user loads a new file.
  // That happens on the graphics thread, upon mouse action.
  MUTEX_LOCK(m_pMutex);     

  // Wrap around (near) end of WAV buffer
  if( m_currSample + m_bufferSize > m_numSamples) {
    m_currSample = 0;
  }
  // Copy 2*nFrames from WAV buffer to working buffer
  memcpy(m_WindowedBuffers[m_winBufIndex], m_sampleBuffer+m_currSample, m_bufferSize*sizeof(Sample));

  // Unlock
  MUTEX_UNLOCK(m_pMutex);


  // Do time domain envelope     
  Sample timeEnvelope = m_timeEnvelope[m_currSample/(m_bufferSize/2)];    
  Sample currTimeEnvelope = leakyIntegrate(lastTimeEnvelopeVal, timeEnvelope);
  for(int i=0; i<m_bufferSize; i++) {         
    m_WindowedBuffers[m_winBufIndex][i] *= currTimeEnvelope;    
    currTimeEnvelope = leakyIntegrate(lastTimeEnvelopeVal, timeEnvelope);
  }

  // Save envelope value
  lastTimeEnvelopeVal = timeEnvelope;


  // Window the signal
  multBuffers(m_WindowedBuffers[m_winBufIndex], m_HanningBuffer, m_bufferSize);

  // Do FFT
  rfft(m_WindowedBuffers[m_winBufIndex], m_bufferSize/2, FFT_FORWARD);


  // Do Frequency and phase envelope
  for(int i=0; i<m_bufferSize/2; i++) {
    // Frequency envelope
    Sample freqEnvelope = m_freqEnvelope[i];
    m_WindowedBuffers[m_winBufIndex][2*i] *= freqEnvelope;
    m_WindowedBuffers[m_winBufIndex][2*i+1] *= freqEnvelope;

    // Phase envelope
    Sample phaseEnvelope = m_phaseEnvelope[i];
    complex val = {m_WindowedBuffers[m_winBufIndex][2*i], m_WindowedBuffers[m_winBufIndex][2*i+1]};
    float magnitude = cmp_abs(val);
    float initialPhase = atan2(val.im, val.re);
    val.re = magnitude*cos(initialPhase+phaseEnvelope*2*MY_PI);
    val.im = magnitude*sin(initialPhase+phaseEnvelope*2*MY_PI);
    m_WindowedBuffers[m_winBufIndex][2*i] = val.re;
    m_WindowedBuffers[m_winBufIndex][2*i+1] = val.im;
  }

  // Lock
  MUTEX_LOCK(m_pMutex);

  // Copy to graphics
  memcpy(fftOutBuf, m_WindowedBuffers[m_winBufIndex], m_bufferSize*sizeof(Sample));

  // Unlock
  MUTEX_UNLOCK(m_pMutex);


  // Do IFFT
  rfft(m_WindowedBuffers[m_winBufIndex], m_bufferSize/2, FFT_INVERSE);

  // Overlap-add and output to RtAudio
  for(int i=0; i<nFrames; i++) {
    Sample samp = m_WindowedBuffers[m_winBufIndex][i] + m_WindowedBuffers[(m_winBufIndex+1)%2][i+nFrames];
    // Copy to audio output
    *pOutputSample  = samp;
    //assert(samp<1.01);
    pOutputSample++;
  }

  // Toggle buffer index between zero and one
  m_winBufIndex = (m_winBufIndex+1)%2;

  //lastFreqEnvelopeVal = 

  m_currSample += readCount/2;

}

void AudioEngine::loadWavFile(const char* fileName)
{
  FileWvIn inFile;
  // Open input file
  inFile.openFile(fileName);

  // Set file read rate
  inFile.setRate(1);

  m_numSamples = inFile.getSize();

  // Lock
  MUTEX_LOCK(m_pMutex);

  delete m_sampleBuffer;
  m_sampleBuffer = new StkFloat[m_numSamples];
  m_numWindows = 2 * m_numSamples / m_bufferSize;
  m_currSample = 0;
  memset(m_sampleBuffer, 0, m_numSamples*sizeof(StkFloat));

  delete m_timeEnvelope;
  m_timeEnvelope = new Sample[m_numWindows];
  initEnvelope(m_timeEnvelope, 1.0, m_numWindows);

  int i = 0;
  // Load file into buffer
  while(!inFile.isFinished() && i < m_numSamples) {
    StkFloat out = inFile.tick();
    //cout << out << endl;
    *(m_sampleBuffer+i) = out;
    i++;
  }

  // Unlock
  MUTEX_UNLOCK(m_pMutex);

} 

void  AudioEngine::setRange(float start, float end, int type)
{
  m_lastRangeStart = start;
  m_lastRangeEnd = end;
  m_lastRangeType = type;
}

void AudioEngine::setEnvelope(int type, list<point>& envelopeInput,
                              float maxX, float minX, 
                              float maxY, float minY)
{
  float* pEnv = NULL;
  int length = 0;
  float rangeStart=0, rangeEnd=1.0;

  switch(m_lastRangeType) {
    case TYPE_TIME:
      m_ranges[type].time.start = m_lastRangeStart;
      m_ranges[type].time.end = m_lastRangeEnd;
      break;
    case TYPE_FREQ:
    case TYPE_PHASE:
      m_ranges[type].freq.start = m_lastRangeStart;
      m_ranges[type].freq.end = m_lastRangeEnd;
      break;
    default:
      break;
  }


  switch(type) {
    case TYPE_TIME:
      pEnv = m_timeEnvelope;
      length = m_numWindows;
      rangeStart = m_ranges[TYPE_TIME].time.start;
      rangeEnd = m_ranges[TYPE_TIME].time.end;

      break;
    case TYPE_FREQ:
      pEnv = m_freqEnvelope;
      length = m_bufferSize/2;
      rangeStart = m_ranges[TYPE_FREQ].freq.start;
      rangeEnd = m_ranges[TYPE_FREQ].freq.end;
      break;
    case TYPE_PHASE:
      pEnv = m_phaseEnvelope;
      length = m_bufferSize/2;
      break;

    default:
      assert(0);
  }

  calcEnvelope(pEnv, length, envelopeInput, maxX, minX, maxY, minY,
               rangeStart, rangeEnd);
}

void AudioEngine::calcEnvelope(float* pEnvelope, int length,  list<point>& envelopeInput,
                               float env_maxX, float env_minX,
                               float env_maxY, float env_minY,
                               float rangeStart, float rangeEnd)

{
  int numPoints = envelopeInput.size();
  int startIndex = (int)floor(rangeStart*length);
  //int endIndex;
  int applyLength = (int)floor((rangeEnd - rangeStart)*length);

  // Reset envelope
  for(int x=0; x<length; x++) {
    pEnvelope[x] = INVALID_ENV;
  }

  if (numPoints >=1) {
    list<point>::iterator iter1 = envelopeInput.begin();
    point p = *iter1;

    float maxY = p.y;
    float minY = p.y;
    float maxX = p.x;
    float minX = p.x;
    for( ; iter1!= envelopeInput.end(); ++iter1 ) {
      point p = *iter1;
      if(p.y > maxY) {
        maxY = p.y;
      }
      if (p.y < minY) {
        minY = p.y;
      }
      if (p.x > maxX) {
        maxX = p.x;
      }
      if (p.x < minX) {
        minX = p.x;
      }
    }

    // Limit min and max
    minX = MAX(minX,env_minX);
    maxX = MIN(maxX,env_maxX);

    // Apply envelope 
    float xinc = applyLength /(float)numPoints;
    float currX = 0.0;
    int currIndex = 0.0;
    int ctr =0;
    for( list<point>::iterator  iter2 = envelopeInput.begin(); iter2!= envelopeInput.end(); ++iter2 ) {
      ctr++;
      p = *iter2;
      // find x index
      float index = ((p.x - minX)*(applyLength-1) / (maxX - minX)) + startIndex;
      // round to closest int
      int envIndex = int(floor((index)+0.5));
      if (envIndex < 0 || envIndex >= length) {
        continue;
      }

      float env = (p.y - env_minY) / (env_maxY - env_minY);

      env = MAX(0,env);
      assert(envIndex < length);
      pEnvelope[envIndex] = env;
      if(fabs(pEnvelope[envIndex]) > 3)
        cerr << "this is weird" << pEnvelope[envIndex] << endl;

    }
  }


  // Linearly interpolate the missing data in the envelope
  int i=startIndex, j=startIndex;
  float prev=0.0, next = 0.0;
  while(i<applyLength+startIndex) {
    prev = pEnvelope[i];
    j=i+1;
    // Find next invalid value
    while((j<applyLength+startIndex) && 
      (pEnvelope[j] == INVALID_ENV)) {
        j++;
    }

    if(j == applyLength+startIndex) {
      // There are no more valid values, use last valid value
      // for all remaining points
      next = prev;
    } else {
      next = pEnvelope[j];
    }

    // Fill in missing points between i and j using linear interpolation
    float slope = (next - prev) / (j-i);
    for(int k=i; k<j; k++) {
      pEnvelope[k] = prev + (k-i)*slope;
      if(fabs(pEnvelope[k]) > 3)
        cerr << "this is weird" << pEnvelope[k] << endl;
    }
    i = j;
  }


  for (int i=0; i<length; i++) {
    if(pEnvelope[i] == INVALID_ENV) {
      pEnvelope[i] = 1.0;
    }
  }
}

void AudioEngine::initEnvelope(Sample* pBuf, Sample value, int len)
{
  for (int i=0; i<len; i++) {
    pBuf[i] = value;
  }
}

float AudioEngine::leakyIntegrate(float oldVal, float newVal)
{
  return LEAK*oldVal + (1.0-LEAK)*newVal;
}

void AudioEngine::multBuffers(Sample* bufIn, Sample* bufMult, long length)
{
  for(int i =0; i<length; i++) {
    bufIn[i] = bufIn[i]*bufMult[i];
  }
}
