////////////////////////////////////////////////////////////////////////////
// File Name:   snd_explorer.cpp
// Description: Sound Explorer proejct main.
// Author:      Roy Fejgin
// Date:        Autumn 2009
////////////////////////////////////////////////////////////////////////////

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include <signal.h>
#include <assert.h>
#include <iostream>
#include <string.h>
#include <list>


// STK
#include "Stk.h"
#include "FileWvIn.h"

// Audicle / Tapestrea
#include "audicle_utils.h"

// Project includes
#include "RtAudio.h"
#include "chuck_fft.h"
#include "ui_util.h"
#include "ui_element.h"
#include "ui_envelope.h"
#include "ui_waterfall.h"
#include "ui_button.h"
#include "ui_range.h"
#include "audio_engine.h"
#include "Common.h"

/*********************************************************************************
Macros
**********************************************************************************/
#ifdef WIN32
#define SAMPLE_RATE     (44100)
#define BUFFER_FRAMES   (1024)
#else
#define SAMPLE_RATE     (48000)
#define BUFFER_FRAMES   (512)
#endif

#define OUT_CHANNELS    (1)
#define IN_CHANNELS     (1)

#define FFT_HISTORY_LEN (70)
#define X_START         (-2.0f)
#define Y_START         (1.8f)

// scaling of the output's amplitude 
#define AMPLITUDE       (1.0)
#define INVALID_ENV     (-1.0)

/*********************************************************************************
Type Definitions
*********************************************************************************/
typedef enum ElementID {
  ELEM_ENVELOPE_FREQ = 0,
  ELEM_ENVELOPE_PHASE,
  ELEM_ENVELOPE_TIME,
  ELEM_WATERFALL,
  ELEM_LOAD_BUTTON,
  ELEM_PHASE_BUTTON,
  ELEM_RANGE,
  ELEM_LAST
} ElementID;

/*********************************************************************************
Globals
*********************************************************************************/
using namespace std;        
using namespace stk;        

///////////////////////////////////////////////////////////////////////////
/// GUI 
///////////////////////////////////////////////////////////////////////////
// Width and height of the window
GLsizei g_width = 1200;
GLsizei g_height = 768;

// Set to true to freeze display
bool g_freeze = false;

// GUI elements
UI_Element* elements[ELEM_LAST];

// Element the mouse interacting with.
UI_Element* g_pCurElement = NULL;

// Active envelope
ElementID g_activeEnv = ELEM_ENVELOPE_TIME;

// Waterfall
UI_Waterfall* g_pWaterfall = NULL;

///////////////////////////////////////////////////////////////////////////
/// Audio
///////////////////////////////////////////////////////////////////////////
AudioEngine* g_pEngine = NULL;

// FFT result. Used to transfer from Audio -> Graphics thread
Sample* g_buffer = NULL;
int g_bufferSize = 0;

// Global RtAudio object
RtAudio * pAudio = NULL;
int g_sampleRate = 0;


// Mutex for synchronizing between audio and graphics threads
OsMutex g_audioMutex;


/*********************************************************************************
Function Prototypes
**********************************************************************************/
void dumpBuffer(Sample* pBuf, int len);
void initUiElements();
void loadFileCallback(void);
void randCallback(void);
void setActiveEnvelope(int type);

// OpenGL callbacks
void idleFunc( );
void displayFunc( );
void reshapeFunc( GLsizei width, GLsizei height );
void keyboardFunc( unsigned char, int, int );
void mouseFunc( int button, int state, int x, int y );
void motionFunc( int x, int y);

/*********************************************************************************
 Function Implementations
**********************************************************************************/
////////////////////////////////////////////////////////////////
// Function Name: initUiElements
// Description: Creates, places and initializes all objects
//              forming the GUI.
// Parameters: 
// Returns:
//////////////////////////////////////////////////////////////////
void initUiElements()
{
  Color c = Color(YELLOW(1.0));
  elements[ELEM_ENVELOPE_TIME] = new UI_Envelope(*g_pEngine, 0.5, 1.9, -1.8, -0.2,
                                                 true, c, AudioEngine::TYPE_TIME,
                                                 "Time","Amplitude");

  c = Color(RED(1.0));
  elements[ELEM_ENVELOPE_FREQ] = new UI_Envelope(*g_pEngine, 0.5, 1.9, -1.8, -0.2,
                                                 false, c, AudioEngine::TYPE_FREQ,
                                                 "Frequency","Magnitude");

  c = Color(PURPLE(1.0));
  elements[ELEM_ENVELOPE_PHASE] = new UI_Envelope(*g_pEngine, 0.5, 1.9, -1.8, -0.2,
                                                 false, c, AudioEngine::TYPE_PHASE,
                                                 "Frequency","Phase");

  c = Color(YELLOW(1.0));
  elements[ELEM_RANGE] = new UI_Range(*g_pEngine, -1.9, -0.5, -1.8, -0.2,
                                                 true, c, AudioEngine::TYPE_TIME,
                                                 "Time","Amplitude");



  elements[ELEM_WATERFALL] = new UI_Waterfall(X_START, Y_START, true, FFT_HISTORY_LEN, g_bufferSize);
  g_pWaterfall = (UI_Waterfall*)elements[ELEM_WATERFALL];

  elements[ELEM_LOAD_BUTTON] = new UI_Button(-.23,-1.8, 0.36, 0.25, Color(WHITE(1.0)), Color(BLACK(1.0)),
                                             "Load File", loadFileCallback);

  elements[ELEM_PHASE_BUTTON] = new UI_Button(-.23,-1.3, 0.36, 0.25, Color(WHITE(1.0)), Color(BLACK(1.0)),
                                             "Random", randCallback);
}

////////////////////////////////////////////////////////////////
// Function Name: AudioCallback
// Description: RtAudio callback
// Parameters: 
// Returns:
//////////////////////////////////////////////////////////////////
int AudioCallback( void *outputBuffer, void *inputBuffer,
                  unsigned int nFrames,
                  double streamTime,
                  RtAudioStreamStatus status,
                  void *userData )
{
   if (status == 0) {
     g_pEngine->renderAudio((Sample*)outputBuffer, (Sample*)inputBuffer, g_buffer, nFrames);
   }
   else {
     cerr << "Overflow/Underflow occured, status is " << status << endl;
   }

  return 0;
}

////////////////////////////////////////////////////////////////
// Function Name: loadFileCallback
// Description: Called when the 'load file' button is pressed
//              and brings up the load file dialog.
// Parameters: 
// Returns:
//////////////////////////////////////////////////////////////////
void loadFileCallback(void)
{
  DirScanner dialog;
  fileData * pFileData;
  pFileData = dialog.openFileDialog();

  if(pFileData != NULL) {
    g_pEngine->loadWavFile(pFileData->fileName.c_str());
  }
}

////////////////////////////////////////////////////////////////
// Function Name: loadFileCallback
// Description: Called when the 'random' button is pressed
//              and causes randomization of the current envelope.
// Parameters: 
// Returns:
//////////////////////////////////////////////////////////////////
void randCallback(void)
{
  ((UI_Envelope*)elements[g_activeEnv])->randomize();
}

////////////////////////////////////////////////////////////////
// Function Name: cleanUp
// Description: Tries to close the audio stream and release the RtAudio object
//              
// Parameters: none
// Returns: void
////////////////////////////////////////////////////////////////
void cleanUp (void)
{
  DEBUG_PRINTF("cleanup: in\n");
  try {    
    if (pAudio) {// cleanup RtAudio
      pAudio->closeStream();

      // TODO cleanup the rest...
    }
  } catch (...) {
    cerr << "Exception Caught!" << endl;
  }
  DEBUG_PRINTF("cleanup: out\n");
}

////////////////////////////////////////////////////////////////
// Function Name: idToType
// Description: Convert element ID to Audio Engine type
//              
// Parameters: 
// Returns:
//////////////////////////////////////////////////////////////////
int idToType(ElementID id) 
{
  switch(id) {
    case ELEM_ENVELOPE_FREQ:
      return AudioEngine::TYPE_FREQ;
      break;
    case ELEM_ENVELOPE_TIME:
      return AudioEngine::TYPE_TIME;
      break;
    case ELEM_ENVELOPE_PHASE:
      return AudioEngine::TYPE_PHASE;
      break;
  }
  return -1;
}

////////////////////////////////////////////////////////////////
// Function Name: signalHandler
// Description: Cleans up and exits
// Parameters:
// Returns:
////////////////////////////////////////////////////////////////
void signalHandler(int sig)
{
  cleanUp();
  exit(1);
}

void printWelcome()
{
  cout << "\nWelcome to Sound Explorer.\n\n"
    "May your explorations go well!" << endl;
}

////////////////////////////////////////////////////////////////
// Function Name: main
// Description: Program main
// Parameters:
// Returns:
////////////////////////////////////////////////////////////////
int main( int argc, char ** argv )
{       
  unsigned int bufferFrames = BUFFER_FRAMES;
  // initialize GLUT
  glutInit( &argc, argv );
  // double buffer, use rgb color, enable depth buffer
  glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH );
  // initialize the window size
  glutInitWindowSize( g_width, g_height );
  // set the window postion
  glutInitWindowPosition( 100, 100 );
  // create the window
  glutCreateWindow( "Sound Explorer" );

  // set the idle function - called when idle
  glutIdleFunc( idleFunc );
  // set the display function - called when redrawing
  glutDisplayFunc( displayFunc );
  // set the reshape function - called when client area changes
  glutReshapeFunc( reshapeFunc );
  // set the keyboard function - called on keyboard events
  glutKeyboardFunc( keyboardFunc );
  // set the mouse function - called on mouse stuff
  glutMouseFunc( mouseFunc );  
  glutMotionFunc( motionFunc);
  glEnable( GL_DEPTH_TEST );
 

  // Create a menu and connect it to the menuHandler function
  int menu = glutCreateMenu(setActiveEnvelope);

  // Add entries to menu
  glutAddMenuEntry("Frequency", ELEM_ENVELOPE_FREQ);
  glutAddMenuEntry("Time", ELEM_ENVELOPE_TIME);
  glutAddMenuEntry("Phase", ELEM_ENVELOPE_PHASE);

  // Attach menu to mouse
  glutAttachMenu(GLUT_RIGHT_BUTTON);

  // Register some signal handlers to allow clean exiting.
  (void) signal(SIGINT, signalHandler);
  (void) signal(SIGABRT, signalHandler);
  (void) signal(SIGTERM, signalHandler);

  // Initialize the mutex
  MUTEX_INITIALIZE(&g_audioMutex);

  /* Initialize audio */
  try {
    RtAudio::StreamParameters outputParameters, inputParameters;


    pAudio = new RtAudio( RT_AUDIO_API_TYPE );

    /* ---- Initialize input and output parameters ------------------- */
    inputParameters.firstChannel = 0;
    inputParameters.nChannels = IN_CHANNELS; 
    outputParameters.firstChannel = 0;
    outputParameters.nChannels = OUT_CHANNELS;
#if defined(__WINDOWS_DS__) || defined(__WINDOWS_ASIO__)        
    outputParameters.deviceId = 0;
    inputParameters.deviceId = 0;
#else // Linux or Mac
    outputParameters.deviceId = pAudio->getDefaultOutputDevice();
    inputParameters.deviceId = pAudio->getDefaultInputDevice();
#endif
    /* ---------------------------------------------------------------- */

    // Open the audio stream
    pAudio->openStream( &outputParameters,
      &inputParameters,
      RTAUDIO_FLOAT32,
      SAMPLE_RATE,
      &bufferFrames, 
      AudioCallback,
      NULL);

    g_bufferSize = bufferFrames*2;

    // allocate buffers and zero initialize them
    g_buffer = new Sample[g_bufferSize];
    memset(g_buffer, 0, bufferFrames*sizeof(Sample));

    // Initialize STK
    g_sampleRate = pAudio->getStreamSampleRate();
    Stk::setSampleRate( g_sampleRate );

    
    try {
      char* fileName = "oscs2.wav";
      g_pEngine = new AudioEngine(g_bufferSize, &g_audioMutex, fileName);
    }
    catch( StkError & e ) {
      cerr << "Error opening file!" << endl;
      return 1;
    }



    // Start the stream
    pAudio->startStream();
  } catch (RtError & err) {
    err.printMessage();
    goto cleanup;
  }

  // Initialize our UI
  initUiElements();

  printWelcome();

  // let GLUT handle the current thread from here
  glutMainLoop();

  // if we get here, then stop!
  try {
    pAudio->stopStream();
  } catch( RtError & err ) {
    // do stuff
    err.printMessage();
  }

cleanup:
  pAudio->stopStream();
  delete pAudio;
  delete g_buffer;

  return 0;
}

////////////////////////////////////////////////////////////////
// Function Name: reshapeFunc
// Description: Called when window size changes
// Parameters:
// Returns:
////////////////////////////////////////////////////////////////
void reshapeFunc( GLsizei w, GLsizei h )
{
  // save the new window size
  g_width = w; g_height = h;
  // map the view port to the client area
  glViewport( 0, 0, w, h );
  // set the matrix mode to project
  glMatrixMode( GL_PROJECTION );
  // load the identity matrix
  glLoadIdentity( );
  // create the viewing frustum
  gluPerspective(45.0, (GLfloat) w / (GLfloat) h, 1.0, 3000.0 );
  //gluOrtho2D(-2,2,-2,2);

  // set the matrix mode to modelview
  glMatrixMode( GL_MODELVIEW );
  // load the identity matrix
  glLoadIdentity( );
  // position the view point
  gluLookAt( 0.0f, 3.0f, 4.0f, 0.0f, 0.0f, -3.0f, 0.0f, 1.0f, 0.0f );
}

////////////////////////////////////////////////////////////////
// Function Name: setActiveEnvelope
// Description:
// Parameters:
// Returns:
////////////////////////////////////////////////////////////////
void setActiveEnvelope(int type)
{
  int i;
  int j = ELEM_WATERFALL;
  for(i=0; i<ELEM_WATERFALL; i++) {
    elements[i]->setActive(i==type);
    if(i == type) {
      j = i;
      g_activeEnv = (ElementID)i;
    }
  }
  if(j<ELEM_WATERFALL) {
    ((UI_Range*)elements[ELEM_RANGE])->setXlabel(((UI_Envelope*)(elements[j]))->getXlabel());
    ((UI_Range*)elements[ELEM_RANGE])->clear();
    ((UI_Range*)elements[ELEM_RANGE])->setType(idToType((ElementID)j));
  }
}

////////////////////////////////////////////////////////////////
// Function Name: keyboardFunc
// Description: Key event handler
// Parameters:
// Returns:
////////////////////////////////////////////////////////////////
void keyboardFunc( unsigned char key, int x, int y )
{
  switch( key )
  {
  case ' ':
    // Toggle display freeze
    g_freeze = !g_freeze;
    break;

  case 'f':
  case 'F':
    // Frequency envelope input
    setActiveEnvelope(ELEM_ENVELOPE_FREQ);
    break;

  case 'p':
  case 'P':
    // Phase envelope input
    setActiveEnvelope(ELEM_ENVELOPE_PHASE);
    break;

  case 't':
  case 'T':
    // Time envelope input
    setActiveEnvelope(ELEM_ENVELOPE_TIME);
    break;

  case 'Q':
  case 'q':
    exit(0);
    break;
  }

  glutPostRedisplay( );
}

////////////////////////////////////////////////////////////////
// Function Name: motionFunc
// Description: Mouse motion handler
// Parameters:
// Returns:
////////////////////////////////////////////////////////////////
void motionFunc( int x, int y)
{
  // ignore backwards mouse motion
  
    point p;
    p.x = x;
    p.y = y;

    // Unproject
    switchToOrtho();    
    p = getPos(p.x,p.y);
    switchToPerspective();

    if (g_pCurElement != NULL) {
      g_pCurElement->mouseMotion(p);
    }

    glutPostRedisplay();
    return;
}

////////////////////////////////////////////////////////////////
// Function Name: mouseFunc
// Description: Mouse event handler
// Parameters:
// Returns:
////////////////////////////////////////////////////////////////
void mouseFunc( int button, int state, int x, int y )
{
  if( button == GLUT_LEFT_BUTTON )
  {
    point p = {x,y};
    // Unproject
    switchToOrtho();    
    p = getPos(p.x,p.y);
    switchToPerspective();

    // When left mouse button is down
    if( state == GLUT_DOWN )
    {
      // Activate the UI element under the mouse
      for(int i=0; i<ELEM_LAST; i++) {
        if(elements[i]->isActive() &&
          elements[i]->intersect(p)) {
            elements[i]->mouseDown(p);
            g_pCurElement  = elements[i];         
            break;
        }
      }
    }
    // Mouse up
    else
    {
      if (g_pCurElement != NULL) {
        g_pCurElement->mouseUp(p);
      }
    }
  }

  glutPostRedisplay( );
}

////////////////////////////////////////////////////////////////
// Function Name: idleFunc
// Description: Doesn't do much :)
// Parameters:
// Returns:
////////////////////////////////////////////////////////////////
void idleFunc( )
{
  // render the scene
  glutPostRedisplay( );
}

////////////////////////////////////////////////////////////////
// Function Name: displayFunc
// Description: Draws the graphics
//
// Parameters:
// Returns:
////////////////////////////////////////////////////////////////
void displayFunc( )
{
  if (g_freeze) {
    // Flush
    glFlush();
    // Swap the double buffer
    glutSwapBuffers( );
    return;
  }

  MUTEX_LOCK(&g_audioMutex);
  // ---- Start critical section -----------------------

  // Copy FFT results into waterfall
  g_pWaterfall->insertSpectrum(g_buffer);

  // ---- End critical section -----------------------
  MUTEX_UNLOCK(&g_audioMutex);


  // Clear the color and depth buffers
  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

  // Draw *everything* ----------------
  for(int i=0; i<ELEM_LAST; i++) {
    if(elements[i]->isActive()) {
      elements[i]->draw();
    }
  }

  // Flush
  glFlush();
  // Swap the double buffer
  glutSwapBuffers( );
}
